import 'dart:async';

import 'package:flutter/services.dart';

abstract class MDNSPluginDelegate {
  void onDiscoveryStarted();
  void onDiscoveryStopped();
  void onServiceFound(String name);
  void onServiceResolved(String name);
  void onServiceRemoved(String name);
}


class MDNSService {
  final Map map;

  MDNSService.fromMap(this.map) : assert(map != null);

  // PROPERTIES
  String get name => map["name"];
  String get hostName => map["hostName"];
  String get serviceType => map["type"];
  int get port => map["port"];
  Map get txt => map["txt"];

  // STRINGIFY
  String toString() => "<MDNSService>{ name=$name serviceType=$serviceType host=$hostName:$port txt=$txt }";
}

class MDNSPlugin {
  static const MethodChannel _channel = const MethodChannel('mdns_plugin');
  final EventChannel _delegate = const EventChannel('mdns_plugin_delegate');

  MDNSPlugin() {
    _delegate.receiveBroadcastStream().listen((args) {
      if(args is Map && args.containsKey("method")) {
        switch(args["method"]) {
          case "onDiscoveryStarted":
            print("onDiscoveryStarted");
            break;
          case "onDiscoveryStopped":
            print("onDiscoveryStopped");
            break;
          case "onServiceFound":
            var service = MDNSService.fromMap(args);
            print("onServiceFound $service");
            break;
          case "onServiceResolved":
            var service = MDNSService.fromMap(args);
            print("onServiceResolved $service");
            break;
          case "onServiceRemoved":
            var service = MDNSService.fromMap(args);
            print("onServiceRemoved $service");
            break;
        }
      }
    });
  }

  static Future<String> get platformVersion async {
    final String version = await _channel.invokeMethod('getPlatformVersion');
    return version;
  }

  Future<void> startDiscovery(String serviceType,{String domain}) async {
    return await _channel.invokeMethod("startDiscovery",{ "serviceType": serviceType, "domain": domain });
  }

  Future<void> stopDiscovery() async {
    return await _channel.invokeMethod('stopDiscovery',{ });
  }
}
